"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var type_1 = require("./type");
var package_1 = __importDefault(require("./package"));
var commandKeyWords = {
    help: {
        isMandatory: false,
        keyword: '--help',
    },
    version: {
        isMandatory: false,
        keyword: '--version',
    },
    dir: {
        isMandatory: true,
        keyword: '--dir',
    },
    include: {
        isMandatory: false,
        keyword: '--include',
    },
    showChanges: {
        type: 'deprecated',
        isMandatory: false,
        keyword: '--showchanges',
        reason: 'The function showchanges is deprecated in favor of `showprorgess`',
    },
    assignment: {
        assign: '=',
    },
};
var TokenParser = /** @class */ (function () {
    function TokenParser(token) {
        var _this = this;
        this.token = token;
        this.parseVersion = function () {
            if (_this.token !== commandKeyWords.version.keyword) {
                return {
                    status: 'no',
                };
            }
            return {
                status: 'yes',
                type: 'version',
                value: package_1.default.version,
            };
        };
        this.parseHelp = function () {
            if (_this.token !== commandKeyWords.help.keyword) {
                return {
                    status: 'no',
                };
            }
            return {
                status: 'yes',
                type: 'help',
                value: fs_1.default.readFileSync('public/help.md', { encoding: 'utf-8' }),
            };
        };
        this.parseDir = function () {
            var _a = _this.token.split(commandKeyWords.assignment.assign), dirWord = _a[0], dirTarget = _a[1];
            if (dirWord !== commandKeyWords.dir.keyword) {
                return {
                    status: 'no',
                };
            }
            return {
                status: 'yes',
                type: 'dir',
                value: (0, type_1.guard)({
                    value: dirTarget,
                    error: function () {
                        return new Error("There is no directory to target since ".concat(dirWord, "={empty}"));
                    },
                }),
            };
        };
        this.parseInclude = function () {
            var _a = _this.token.split(commandKeyWords.assignment.assign), includeWord = _a[0], includeTarget = _a[1];
            if (includeWord !== commandKeyWords.include.keyword) {
                return {
                    status: 'no',
                };
            }
            return {
                status: 'yes',
                type: 'include',
                value: (0, type_1.guard)({
                    value: includeTarget,
                    error: function () {
                        return new Error("There is no include directories to target since ".concat(includeWord, "={empty}"));
                    },
                }).split(','),
            };
        };
        this.processShowChanges = function () {
            var _a = _this.token.split(commandKeyWords.assignment.assign), includeWord = _a[0], includeTarget = _a[1];
            if (includeWord !== commandKeyWords.showChanges.keyword) {
                return {
                    status: 'no',
                };
            }
            if (includeTarget === 'true' || includeTarget === 'false') {
                return {
                    status: 'yes',
                    type: 'showChanges',
                    value: JSON.parse(includeTarget),
                };
            }
            throw new Error("".concat(includeWord, "=").concat(includeTarget, " is invalid, it can only receive boolean value"));
        };
        this.processNonRecognisableToken = function () {
            return {
                status: 'invalid',
                reason: 'then token cannot be recognized',
                token: _this.token,
            };
        };
    }
    return TokenParser;
}());
var ParseArgs = /** @class */ (function () {
    function ParseArgs(tokens) {
        var _this = this;
        this.tokens = tokens;
        this.unifyAssignment = function () {
            return _this.tokens.flatMap(function (args, index, self) {
                var assign = commandKeyWords.assignment.assign;
                if (args.startsWith('--')) {
                    if (args.includes(assign)) {
                        return [args];
                    }
                    var next = self.at(index + 1);
                    if (next && !(next === null || next === void 0 ? void 0 : next.startsWith('--'))) {
                        return ["".concat(args).concat(assign).concat(next)];
                    }
                }
                return [];
            });
        };
        this.asVersion = function () {
            var version = _this.tokens.reduce(function (result, token) {
                if ((result === null || result === void 0 ? void 0 : result.status) === 'yes') {
                    return result;
                }
                return new TokenParser(token).parseVersion();
            }, { status: 'no' });
            if (version.status === 'no') {
                return {
                    proceed: false,
                };
            }
            return {
                proceed: true,
                value: version.value,
            };
        };
        this.asHelp = function () {
            var help = _this.tokens.length === 1
                ? {
                    status: 'yes',
                    type: 'help',
                    value: fs_1.default.readFileSync('public/help.md', {
                        encoding: 'utf-8',
                    }),
                }
                : _this.tokens.reduce(function (result, token) {
                    if ((result === null || result === void 0 ? void 0 : result.status) === 'yes') {
                        return result;
                    }
                    return new TokenParser(token).parseHelp();
                }, { status: 'no' });
            if (help.status === 'no') {
                return {
                    proceed: false,
                };
            }
            return {
                proceed: true,
                guide: help.value,
            };
        };
        this.asOperation = function () {
            var _b, _c, _d;
            var processedToken = _this.unifyAssignment().map(function (token) {
                var parser = new TokenParser(token);
                var dir = parser.parseDir();
                if (dir.status === 'yes') {
                    return dir;
                }
                var include = parser.parseInclude();
                if (include.status === 'yes') {
                    return include;
                }
                var showChanges = parser.processShowChanges();
                if (showChanges.status === 'yes') {
                    return showChanges;
                }
                return parser.processNonRecognisableToken();
            });
            processedToken
                .flatMap(function (node) { return (node.status !== 'invalid' ? [] : [node]); })
                .forEach(function (node) {
                return console.log("The \"".concat(node.token, "\" in the command is invalid as ").concat(node.reason, ". So please remove it"));
            });
            var nodes = processedToken.flatMap(function (node) {
                return node.status !== 'yes' ? [] : [node];
            });
            return {
                dir: (0, type_1.guard)({
                    value: (_b = nodes.find(function (node) { return node.type === 'dir'; })) === null || _b === void 0 ? void 0 : _b.value,
                    error: function () {
                        return new Error('dir is a mandatory field, it should be present to know which dir it should operate on');
                    },
                }),
                // optional
                include: (_c = nodes.find(function (node) { return node.type === 'include'; })) === null || _c === void 0 ? void 0 : _c.value,
                showChanges: (_d = nodes.find(function (node) { return node.type === 'showChanges'; })) === null || _d === void 0 ? void 0 : _d.value,
            };
        };
        this.tokens = ParseArgs.purifyTokens(tokens);
    }
    var _a;
    _a = ParseArgs;
    ParseArgs.create = function (tokens) {
        var name = ParseArgs.checkPackageName(tokens.at(1));
        if (name !== 'proceed') {
            throw name.error;
        }
        if (tokens.includes('add')) {
            console.log('The "add" in the command can be removed, as it is only used for backward compatibility');
        }
        return new _a(tokens
            .map(function (token) { return (token === 'add' ? '' : token); })
            .filter(function (_, index) { return index; }));
    };
    ParseArgs.checkPackageName = function (name) {
        var _b;
        return (name === null || name === void 0 ? void 0 : name.includes((_b = package_1.default.name) !== null && _b !== void 0 ? _b : ''))
            ? 'proceed'
            : {
                error: new Error("The pkg name \"".concat(name, "\" passed is invalid")),
            };
    };
    ParseArgs.purifyTokens = function (tokens) {
        return tokens.map(function (token) { return (token === 'add' ? '' : token); }).filter(Boolean);
    };
    return ParseArgs;
}());
exports.default = ParseArgs;
//# sourceMappingURL=cli-command-parser.js.map